var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
import { Api } from "../../Api";
import { DataFrame, Method, Status } from "../../DataFrame";
import { ControllerExtensionsService } from "../Controller/ControllerExtensionsService";
import { DevicesService } from "../Devices/DevicesService";
import { ResponseResult } from "../FieldChangeResult";
import { UpdatesProvider } from "./UpdatesProvider";
import { ControllerUpdatesInfo, ControllerUpdateType, ControllerVersion } from "./IUpdateInfo";
import { ProviderType, UpdateInfo, UpdateTypes } from "./IUpdatesProvider";
import { UpdateErrorCode, UpdateProgressInfo, UpdateState, UpdateStatus, UserInput } from "./IUpdatesService";
import { AppStateService } from "../AppStateService";
import { LoggerService } from "../Logging/LoggerService";
import { ExalusConnectionService } from "../ExalusConnectionService";
import { Helpers } from "../../Helpers";
export class UpdatesService {
    constructor() {
        this._controllerUpdates = [];
        this._updateRequestTimeout = 120000;
        this._updateProviders = Api.Get(UpdatesProvider.ServiceName);
        this._logger = Api.Get(LoggerService.ServiceName);
        this._connection = Api.Get(ExalusConnectionService.ServiceName);
        this._appStateService = Api.Get(AppStateService.ServiceName);
    }
    GetServiceName() {
        return UpdatesService.ServiceName;
    }
    CheckDeviceVersionAsync(dev) {
        var _a;
        return __awaiter(this, void 0, void 0, function* () {
            let device;
            if (typeof dev == 'string') {
                const result = yield Api.Get(DevicesService.ServiceName).GetDevice(dev);
                if (result != null)
                    device = result;
                else
                    return new ResponseResult(UpdateErrorCode.CannotFindDevice, `Device with specified GUID ${dev} does not exists.`);
            }
            else {
                device = dev;
            }
            if (device.ProtocolGuid == null)
                return new ResponseResult(UpdateErrorCode.UnsupportedDevice, `Device does not contains data about protocol GUID.`);
            const softwareInfo = yield ((_a = (yield this._updateProviders.GetUpdatesProvidersByProtocolAsync(device.ProtocolGuid, ProviderType.Device)).first()) === null || _a === void 0 ? void 0 : _a.GetSoftwareVersionAsync(device));
            if (softwareInfo.Type != null)
                return softwareInfo;
            return softwareInfo;
        });
    }
    CheckDeviceUpdateAsync(dev) {
        var _a;
        return __awaiter(this, void 0, void 0, function* () {
            let device;
            if (typeof dev == 'string') {
                const result = yield Api.Get(DevicesService.ServiceName).GetDevice(dev);
                if (result != null)
                    device = result;
                else
                    return new ResponseResult(UpdateErrorCode.CannotFindDevice, `Device with specified GUID ${dev} does not exists.`);
            }
            else {
                device = dev;
            }
            if (device.ProtocolGuid == null)
                return new ResponseResult(UpdateErrorCode.UnsupportedDevice, `Device does not contains data about protocol GUID.`);
            const updateInfo = yield ((_a = (yield this._updateProviders.GetUpdatesProvidersByProtocolAsync(device.ProtocolGuid, ProviderType.Device)).first()) === null || _a === void 0 ? void 0 : _a.GetAvailableUpdateAsync(device));
            if (updateInfo.Type != null)
                return updateInfo;
            const result = new DeviceUpdate();
            result.CurrentResourceVersion = updateInfo.CurrentResourceVersion;
            result.NewResourceVersion = updateInfo.Update.Version;
            result.UpdatedResourceName = updateInfo.UpdatedResourceName;
            result.UpdateResourceGuid = updateInfo.Update.ResourceGuid;
            result.UpdateDescription = new Map([["pl-PL", updateInfo.Update.Description]]);
            result.Protocol = yield (yield Api.Get(ControllerExtensionsService.ServiceName).GetProtocolInfoAsync(device.ProtocolGuid)).first().Name;
            result.ProtocolGuid = device.ProtocolGuid;
            if ((yield Api.Get(DevicesService.ServiceName).GetDevicesAsync()).any(dev => dev.ModelGuid == "952ddc14-76c4-4f47-b0bc-5d106c1f9b02"))
                result.IsUpdateSafe = false;
            return result;
        });
    }
    InstallDeviceUpdateAsync(dev, updateProgress, updateAction) {
        var _a;
        return __awaiter(this, void 0, void 0, function* () {
            let device;
            if (typeof dev == 'string') {
                const result = yield Api.Get(DevicesService.ServiceName).GetDevice(dev);
                if (result != null)
                    device = result;
                else
                    return new ResponseResult(UpdateErrorCode.CannotFindDevice, `Device with specified GUID ${dev} does not exists.`);
            }
            else {
                device = dev;
            }
            if (device.ProtocolGuid == null)
                return new ResponseResult(UpdateErrorCode.UnsupportedDevice, `Device does not contains data about protocol GUID.`);
            const updateInfo = yield ((_a = (yield this._updateProviders.GetUpdatesProvidersByProtocolAsync(device.ProtocolGuid, ProviderType.Device)).first()) === null || _a === void 0 ? void 0 : _a.GetAvailableUpdateAsync(device));
            if (updateInfo.Type != null)
                return updateInfo;
            this._appStateService.DisallowHibernation();
            const updateInstallResult = yield (yield this._updateProviders.GetUpdatesProvidersByProtocolAsync(device.ProtocolGuid, ProviderType.Device)).first().InstallUpdateAsync(updateInfo, updateProgress, updateAction);
            if (updateInstallResult.Type != null) {
                this._appStateService.AllowHibernation();
                return updateInstallResult;
            }
            this._appStateService.AllowHibernation();
            return Status.OK;
        });
    }
    CheckControllerUpdatesAsync() {
        return __awaiter(this, void 0, void 0, function* () {
            const updateProviderServices = yield this._updateProviders.GetUpdatesProvidersAsync(ProviderType.Controller);
            const updates = new ControllerUpdatesInfo();
            this._controllerUpdates = [];
            //Radio updates
            for (let i = 0; i < updateProviderServices.length; i++) {
                const result = yield updateProviderServices[i].GetAvailableRadioUpdateAsync();
                if (result.Type != null) {
                    const error = new ControllerUpdateNotAvailableInfo();
                    error.ResponseResult = result;
                    error.UpdateType = ControllerUpdateType.RadioUpdate;
                    error.UpdatedResourceName = yield (yield Api.Get(ControllerExtensionsService.ServiceName).GetProtocolInfoAsync(updateProviderServices[i].ProtocolGuid)).first().Name;
                    error.CurrentResourceVersion = yield updateProviderServices[i].GetCurrentRadioVersionAsync();
                    updates.UpdatesNotAvailable.push(error);
                }
                else {
                    const updateInfo = result;
                    const update = new ControllerUpdate();
                    update.UpdatedResourceName = updateInfo.UpdatedResourceName;
                    update.CurrentResourceVersion = updateInfo.CurrentResourceVersion;
                    update.NewResourceVersion = updateInfo.Update.Version;
                    update.UpdateDescription = new Map([["pl-PL", updateInfo.Update.Description]]);
                    update.UpdateType = ControllerUpdateType.RadioUpdate;
                    update.UpdateIdentifier = Helpers.GenerateUUID();
                    updates.UpdatesAvailable.push(update);
                    this._controllerUpdates.push(Object.assign(Object.assign({}, updateInfo), { UpdateIdentifier: update.UpdateIdentifier, ProviderName: updateProviderServices[i].GetUpdateProviderName(), UpdateType: ControllerUpdateType.RadioUpdate }));
                }
            }
            //Software updates
            const runtimeInfo = yield this._updateProviders.GetSoftwareRuntimeInfoAsync();
            const url = `https://updates.tr7.pl/software/${runtimeInfo.UpdateChannel}/${runtimeInfo.RuntimeVersion}/${runtimeInfo.SoftwareVersion}/update.json`;
            try {
                const response = yield fetch(url);
                let updateData;
                let updateInfo = new UpdateInfo();
                const error = new ControllerUpdateNotAvailableInfo();
                error.UpdateType = ControllerUpdateType.SoftwareUpdate;
                error.UpdatedResourceName = "Controller software";
                switch (response.status) {
                    case 200:
                        if (response.body != null)
                            updateData = (yield response.json());
                        break;
                    case 404:
                        error.ResponseResult = new ResponseResult(UpdateErrorCode.DeviceIsUpToDate, `Updates not found, controller is already up-to-date.`);
                        updates.UpdatesNotAvailable.push(error);
                        break;
                    default:
                        error.ResponseResult = new ResponseResult(UpdateErrorCode.CannotGetUpdates, `Response status code does not indicate success. Status: ${response.status}`);
                        updates.UpdatesNotAvailable.push(error);
                        break;
                }
                if (updateData != null) {
                    updateInfo.CurrentResourceVersion = `${runtimeInfo.RuntimeVersion}.${runtimeInfo.SoftwareVersion}`;
                    updateInfo.UpdatedResourceName = "Controller software";
                    updateInfo.Update = updateData;
                    const update = new ControllerUpdate();
                    update.UpdatedResourceName = updateInfo.UpdatedResourceName;
                    update.CurrentResourceVersion = updateInfo.CurrentResourceVersion;
                    update.NewResourceVersion = updateInfo.Update.Version;
                    update.UpdateDescription = new Map([["pl-PL", updateInfo.Update.Description]]);
                    update.UpdateType = ControllerUpdateType.SoftwareUpdate;
                    update.UpdateIdentifier = Helpers.GenerateUUID();
                    updates.UpdatesAvailable.push(update);
                    this._controllerUpdates.push(Object.assign(Object.assign({}, updateInfo), { UpdateIdentifier: update.UpdateIdentifier, ProviderName: UpdatesService.ServiceName, UpdateType: ControllerUpdateType.SoftwareUpdate }));
                }
                else if (response.status == 200)
                    throw (new Error(`Cannot get Update Data from response. Status: ${response.status}`));
            }
            catch (err) {
                const runtimeInfo = (yield this._updateProviders.GetSoftwareRuntimeInfoAsync());
                const error = new ControllerUpdateNotAvailableInfo();
                error.UpdateType = ControllerUpdateType.SoftwareUpdate;
                error.UpdatedResourceName = "Controller software";
                error.ResponseResult = new ResponseResult(UpdateErrorCode.CannotGetUpdates, `Cannot get available updates! ${err}`);
                error.CurrentResourceVersion = `${runtimeInfo.RuntimeVersion}.${runtimeInfo.SoftwareVersion}`;
                updates.UpdatesNotAvailable.push(error);
            }
            return updates;
        });
    }
    CheckControllerVersionAsync() {
        return __awaiter(this, void 0, void 0, function* () {
            const updateProviderServices = yield this._updateProviders.GetUpdatesProvidersAsync(ProviderType.Controller);
            const result = [];
            let version;
            //Radio updates
            for (let i = 0; i < updateProviderServices.length; i++) {
                version = new ControllerVersion();
                version.Version = yield updateProviderServices[i].GetCurrentRadioVersionAsync();
                version.Type = ControllerUpdateType.RadioUpdate;
                version.Resource = yield (yield Api.Get(ControllerExtensionsService.ServiceName).GetProtocolInfoAsync(updateProviderServices[i].ProtocolGuid)).first().Name;
                result.push(version);
            }
            //Software updates
            version = new ControllerVersion();
            version.Resource = "Controller software";
            version.Type = ControllerUpdateType.SoftwareUpdate;
            const runtimeInfo = (yield this._updateProviders.GetSoftwareRuntimeInfoAsync());
            version.Version = `${runtimeInfo.RuntimeVersion}.${runtimeInfo.SoftwareVersion}`;
            result.push(version);
            return result;
        });
    }
    InstallControllerUpdateAsync(param, updateProgress, updateAction) {
        return __awaiter(this, void 0, void 0, function* () {
            const regGuid = new RegExp("^[{]?[0-9a-fA-F]{8}-([0-9a-fA-F]{4}-){3}[0-9a-fA-F]{12}[}]?$");
            const numberOfUpdates = this._controllerUpdates.length;
            //Check if any updates are avaliable
            if (this._controllerUpdates.length == 0)
                return new ResponseResult(UpdateErrorCode.CannotGetUpdates, `Cannot find any updates, make sure you check for available updates by calling CheckControllerUpdateAsync().`);
            //Install single update
            if ((typeof param === 'string') && regGuid.test(param)) {
                let updateResult;
                const updateToInstall = this._controllerUpdates.find(update => update.UpdateIdentifier == param);
                if (updateToInstall == null)
                    return new ResponseResult(UpdateErrorCode.CannotGetUpdates, `Update with requested guid (${param}) not found.`);
                this._logger.Debug(`Installing single update type: ${updateToInstall.Update.UpdateType}, resource: ${updateToInstall.UpdatedResourceName}, version: ${updateToInstall.Update.Version}`);
                this._appStateService.DisallowHibernation();
                switch (updateToInstall.UpdateType) {
                    case ControllerUpdateType.RadioUpdate:
                        updateResult = yield (yield this._updateProviders.GetUpdatesProviderAsync(updateToInstall.ProviderName)).InstallRadioUpdateAsync(updateToInstall, updateProgress, updateAction);
                        this._appStateService.AllowHibernation();
                        if (updateResult == Status.OK)
                            this._controllerUpdates = this._controllerUpdates.filter(update => update.UpdateIdentifier != param);
                        return updateResult;
                    case ControllerUpdateType.SoftwareUpdate:
                        updateResult = yield this.UpdateControllerAsync(updateToInstall.Update, updateProgress);
                        this._appStateService.AllowHibernation();
                        if (updateResult == Status.OK)
                            this._controllerUpdates = this._controllerUpdates.filter(update => update.UpdateIdentifier != param);
                        return updateResult;
                    default:
                        return new ResponseResult(UpdateErrorCode.CannotProcessUpdate, `Update with type ${updateToInstall.UpdateType} is not supported by UpdateService.`);
                }
            }
            //Install all updates
            else {
                //Handle update progress with multiple updates.
                const updateNumberRef = [1];
                const updateProgressWrapper = (progressInfo) => {
                    progressInfo.UpdateNumber = updateNumberRef[0];
                    progressInfo.AvailableUpdates = numberOfUpdates;
                    const callback = param;
                    callback(progressInfo);
                };
                const timeout = (ms) => __awaiter(this, void 0, void 0, function* () {
                    return new Promise(resolve => setTimeout(resolve, ms));
                });
                //First insall radio updates
                let updatesToInstall = this._controllerUpdates.where(update => update.UpdateType == ControllerUpdateType.RadioUpdate).toArray();
                let result = [];
                this._appStateService.DisallowHibernation();
                for (let i = 0; i < updatesToInstall.length; i++) {
                    const updateResult = yield (yield this._updateProviders.GetUpdatesProviderAsync(updatesToInstall[i].ProviderName)).InstallRadioUpdateAsync(updatesToInstall[i], updateProgressWrapper, updateAction);
                    //Wait between updates
                    yield timeout(5000);
                    if (updateResult == Status.OK)
                        this._controllerUpdates = this._controllerUpdates.filter(update => update.UpdateIdentifier != updatesToInstall[i].UpdateIdentifier);
                    result.push(updateResult);
                    updateNumberRef[0]++;
                }
                //Install other updates (in the future)
                //....//
                //
                //Install controller update
                const controlerUpdate = this._controllerUpdates.find(update => update.UpdateType == ControllerUpdateType.SoftwareUpdate);
                if (controlerUpdate != null) {
                    const updateResult = yield this.UpdateControllerAsync(controlerUpdate.Update, updateProgressWrapper);
                    if (updateResult == Status.OK)
                        this._controllerUpdates = this._controllerUpdates.filter(update => update.UpdateIdentifier != controlerUpdate.UpdateIdentifier);
                    result.push(updateResult);
                    updateNumberRef[0]++;
                }
                const resultWithError = result.where(r => r instanceof ResponseResult).toArray();
                if (resultWithError != null && resultWithError.length != 0) {
                    let errors = "";
                    resultWithError.forEach(err => errors += `${err.Type}, `);
                    this._appStateService.AllowHibernation();
                    return new ResponseResult(UpdateErrorCode.UpdatesPartiallyInstalled, `Update installation finished - not all updates were successfully installed. Installation error codes: ${errors}`);
                }
                this._appStateService.AllowHibernation();
                return Status.OK;
            }
        });
    }
    UpdateControllerAsync(update, updateProgress) {
        var _a, _b, _c, _d, _e, _f;
        return __awaiter(this, void 0, void 0, function* () {
            switch (update.UpdateType) {
                case UpdateTypes.Container:
                case UpdateTypes.ContainerSoftware:
                    try {
                        yield ((_a = this._connection) === null || _a === void 0 ? void 0 : _a.SendAndHandleResponseAsync(new UpdateRequest(update.DownloadUri), this._updateRequestTimeout, (resp) => {
                            const progressInfo = new UpdateProgressInfo();
                            progressInfo.UpdatedResourceName = "Controller software";
                            switch (resp.Status) {
                                case Status.MultiDataResponseStart:
                                    progressInfo.Status = UpdateStatus.PreparingForUpgrade;
                                    break;
                                case Status.MultiDataResponse:
                                    if (resp.Data == null) {
                                        this._logger.Warning(UpdatesService.ServiceName, `Recived update response without data! URI: ${update.DownloadUri}`);
                                        return;
                                    }
                                    progressInfo.Percentage = resp.Data.Progress;
                                    switch (resp.Data.State) {
                                        case UpdateState.CheckingApplicability:
                                        case UpdateState.CheckingIntegrity:
                                        case UpdateState.CheckingPackageIntegrity:
                                            progressInfo.Status = UpdateStatus.CheckingPackage;
                                            break;
                                        case UpdateState.CreatingBackup:
                                        case UpdateState.MigratingData:
                                        case UpdateState.Removing:
                                            progressInfo.Status = UpdateStatus.MovingData;
                                            break;
                                        case UpdateState.Deploying:
                                        case UpdateState.Installing:
                                            progressInfo.Status = UpdateStatus.Upgrading;
                                            break;
                                        case UpdateState.Downloading:
                                            progressInfo.Status = UpdateStatus.DownloadingPackage;
                                            break;
                                        case UpdateState.Error:
                                            progressInfo.Status = UpdateStatus.Error;
                                            break;
                                        case UpdateState.Installed:
                                            progressInfo.Status = UpdateStatus.Upgraded;
                                            break;
                                        case UpdateState.PreparingSystem:
                                        case UpdateState.ReadyToDownload:
                                            progressInfo.Status = UpdateStatus.PreparingForUpgrade;
                                            break;
                                        case UpdateState.UndoingInstallation:
                                            progressInfo.Status = UpdateStatus.InstallationStopped;
                                            break;
                                        case UpdateState.Unpackaging:
                                            progressInfo.Status = UpdateStatus.MovingData;
                                            break;
                                        case UpdateState.WaitingForUser:
                                            progressInfo.Status = UpdateStatus.Upgrading;
                                            break;
                                    }
                                    break;
                                case Status.MultiDataResponseStop:
                                    progressInfo.Status = UpdateStatus.OK;
                                    break;
                            }
                            updateProgress(progressInfo);
                        }, false));
                        return Status.OK;
                    }
                    catch (error) {
                        return new ResponseResult(UpdateErrorCode.CannotProcessUpdate, `Cannot process update, error: ${error}`);
                    }
                case UpdateTypes.RuntimeFixingScript:
                    var howManyDevices = 10;
                    var updateExtVersion = (yield Api.Get(ControllerExtensionsService.ServiceName).GetExtensionsInfoAsync()).first(ext => ext.ExtensionGuid == "f4f45dac-beda-441a-8d29-efc6dc977cd1").ExtensionVersion;
                    var isNewApi = parseFloat((yield this._updateProviders.GetSoftwareRuntimeInfoAsync()).RuntimeVersion) >= 2.0 ? true : (parseFloat(updateExtVersion) >= 1.2201125);
                    try {
                        howManyDevices = (yield Api.Get(DevicesService.ServiceName).GetDevicesAsync()).length;
                    }
                    catch (error) {
                        this._logger.Warning(UpdatesService.ServiceName, `Cannot get number of devices in system! Error: ${error}. Using default value of 10 devices.`);
                    }
                    if (isNewApi) {
                        let intervalId;
                        let progressInfo = new UpdateProgressInfo();
                        progressInfo.UpdatedResourceName = "Controller software";
                        progressInfo.Percentage = 1;
                        progressInfo.Status = UpdateStatus.PreparingForUpgrade;
                        updateProgress(progressInfo);
                        let i = 2;
                        const startTime = Date.now();
                        const howLongToUpgrade = (howManyDevices * 900) + 360000;
                        const onePercentInTime = howLongToUpgrade / 100;
                        const willEndAt = startTime + howLongToUpgrade;
                        const updatePromise = new Promise((resolve, reject) => __awaiter(this, void 0, void 0, function* () {
                            var _g;
                            try {
                                const OnReturnedFromSuspensionHandler = () => {
                                    this._appStateService.OnReturnedFromSuspension().Unsubscribe(OnReturnedFromSuspensionHandler);
                                    if (Date.now() > willEndAt) {
                                        clearInterval(intervalId);
                                        this._logger.Debug(`Update already finished!`);
                                        progressInfo.Percentage = 100;
                                        progressInfo.Status = UpdateStatus.Upgraded;
                                        updateProgress(progressInfo);
                                        resolve(Status.OK);
                                    }
                                    else {
                                        var msToEnd = (willEndAt - Date.now());
                                        if (msToEnd < howLongToUpgrade) {
                                            i = (80 - (msToEnd / onePercentInTime));
                                        }
                                        else {
                                            clearInterval(intervalId);
                                            intervalId = setInterval(() => {
                                                clearInterval(intervalId);
                                                progressInfo.Percentage = 100;
                                                progressInfo.Status = UpdateStatus.Upgraded;
                                                updateProgress(progressInfo);
                                                resolve(Status.OK);
                                            }, msToEnd);
                                        }
                                    }
                                };
                                this._appStateService.OnReturnedFromSuspension().Subscribe(OnReturnedFromSuspensionHandler);
                                intervalId = setInterval(() => {
                                    if (i > 99) {
                                        clearInterval(intervalId);
                                        progressInfo.Percentage = 100;
                                        progressInfo.Status = UpdateStatus.Upgraded;
                                        updateProgress(progressInfo);
                                        resolve(Status.OK);
                                    }
                                    else {
                                        var percent = i;
                                        if (percent < 99) {
                                            progressInfo.Percentage = percent;
                                            progressInfo.Status = UpdateStatus.Upgrading;
                                            updateProgress(progressInfo);
                                        }
                                    }
                                    i++;
                                }, onePercentInTime);
                                const result = yield ((_g = this._connection) === null || _g === void 0 ? void 0 : _g.SendAndWaitForResponseAsync(new InstallRequest(update.DownloadUri), this._updateRequestTimeout, false));
                                if (result.Status != Status.OK) {
                                    clearInterval(intervalId);
                                    progressInfo.Percentage = 0;
                                    progressInfo.Status = UpdateStatus.InstallationError;
                                    updateProgress(progressInfo);
                                    this._logger.Debug(UpdatesService.ServiceName, `Update failed with error. Update response status: ${result.Status}`);
                                    resolve(new ResponseResult(UpdateErrorCode.CannotProcessUpdate, `Update failed with error. Update response status: ${result.Status}`));
                                }
                            }
                            catch (error) {
                                this._logger.Debug(UpdatesService.ServiceName, `Update failed with error. ${error}`);
                                progressInfo.Percentage = 0;
                                progressInfo.Status = UpdateStatus.Error;
                                updateProgress(progressInfo);
                                clearInterval(intervalId);
                                resolve(new ResponseResult(UpdateErrorCode.CannotProcessUpdate, `Update failed with error. ${error}`));
                            }
                        }));
                        return yield updatePromise;
                    }
                    else {
                        let progressInfo = new UpdateProgressInfo();
                        progressInfo.UpdatedResourceName = "Controller software";
                        progressInfo.Percentage = 1;
                        progressInfo.Status = UpdateStatus.PreparingForUpgrade;
                        updateProgress(progressInfo);
                        const currentAuthData = this._connection.GetAuthorizationInfo();
                        if (currentAuthData == null)
                            return new ResponseResult(UpdateErrorCode.CannotProcessUpdate, `Cannot process update. Cannot get current controller login credentials!`);
                        const result = yield ((_b = this._connection) === null || _b === void 0 ? void 0 : _b.SendAndWaitForResponseAsync(new LoginUserRequest({ Email: `support@support`, Password: `support@${currentAuthData.SerialNumber}:${currentAuthData.PIN}` }), 12000, false));
                        if (result.Status === Status.OK) {
                            progressInfo.Percentage = 5;
                            progressInfo.Status = UpdateStatus.PreparingForUpgrade;
                            updateProgress(progressInfo);
                            let executeCommandResult = yield ((_c = this._connection) === null || _c === void 0 ? void 0 : _c.SendAndWaitForResponseAsync(new ExecuteCommandRequest("rm /runtime/tmp/update.sh"), 12000, false));
                            if (executeCommandResult.Status == Status.OK) {
                                this._logger.Debug(`Removed previous script`);
                                progressInfo.Percentage = 10;
                                progressInfo.Status = UpdateStatus.PreparingForUpgrade;
                                updateProgress(progressInfo);
                                executeCommandResult = yield ((_d = this._connection) === null || _d === void 0 ? void 0 : _d.SendAndWaitForResponseAsync(new ExecuteCommandRequest(`wget ${update.DownloadUri} -O /runtime/tmp/update.sh`), this._updateRequestTimeout, false));
                                if (executeCommandResult.Status == Status.OK) {
                                    this._logger.Debug(`Downloaded update script`);
                                    progressInfo.Percentage = 15;
                                    progressInfo.Status = UpdateStatus.PreparingForUpgrade;
                                    updateProgress(progressInfo);
                                    try {
                                        executeCommandResult = yield ((_e = this._connection) === null || _e === void 0 ? void 0 : _e.SendAndWaitForResponseAsync(new ExecuteCommandRequest(`chmod a+x /runtime/tmp/update.sh`), 12000, false));
                                        if (executeCommandResult.Status == Status.OK) {
                                            this._logger.Debug(`Executed update script`);
                                            progressInfo.Percentage = 20;
                                            progressInfo.Status = UpdateStatus.Upgrading;
                                            updateProgress(progressInfo);
                                            let i = 0;
                                            let intervalId;
                                            const onePercent = 80 / 100.0;
                                            const startTime = Date.now();
                                            const howLongToUpgrade = (howManyDevices * 900) + 600000; //ms
                                            const onePercentInTime = howLongToUpgrade / 80;
                                            const willEndAt = startTime + howLongToUpgrade;
                                            const updatePromise = new Promise((resolve, reject) => __awaiter(this, void 0, void 0, function* () {
                                                var _h;
                                                try {
                                                    const OnReturnedFromSuspensionHandler = () => {
                                                        this._appStateService.OnReturnedFromSuspension().Unsubscribe(OnReturnedFromSuspensionHandler);
                                                        if (Date.now() > willEndAt) {
                                                            clearInterval(intervalId);
                                                            this._logger.Debug(`Update already finished!`);
                                                            progressInfo.Percentage = 100;
                                                            progressInfo.Status = UpdateStatus.Upgraded;
                                                            updateProgress(progressInfo);
                                                            resolve(Status.OK);
                                                        }
                                                        else {
                                                            var msToEnd = (willEndAt - Date.now());
                                                            if (msToEnd < howLongToUpgrade) {
                                                                i = (80 - (msToEnd / onePercentInTime));
                                                            }
                                                            else {
                                                                clearInterval(intervalId);
                                                                intervalId = setInterval(() => {
                                                                    clearInterval(intervalId);
                                                                    progressInfo.Percentage = 100;
                                                                    progressInfo.Status = UpdateStatus.Upgraded;
                                                                    updateProgress(progressInfo);
                                                                    resolve(Status.OK);
                                                                }, msToEnd);
                                                            }
                                                        }
                                                    };
                                                    this._appStateService.OnReturnedFromSuspension().Subscribe(OnReturnedFromSuspensionHandler);
                                                    intervalId = setInterval(() => {
                                                        if (i > 99) {
                                                            clearInterval(intervalId);
                                                            progressInfo.Percentage = 100;
                                                            progressInfo.Status = UpdateStatus.Upgraded;
                                                            updateProgress(progressInfo);
                                                            resolve(Status.OK);
                                                        }
                                                        else {
                                                            var percent = (20 + (i * onePercent));
                                                            ;
                                                            if (percent < 99) {
                                                                progressInfo.Percentage = percent;
                                                                progressInfo.Status = UpdateStatus.Upgrading;
                                                                updateProgress(progressInfo);
                                                            }
                                                        }
                                                        i++;
                                                    }, onePercentInTime);
                                                    (_h = this._connection) === null || _h === void 0 ? void 0 : _h.SendAndWaitForResponseAsync(new ExecuteCommandRequest(`/runtime/tmp/update.sh`), 12000, false);
                                                }
                                                catch (error) {
                                                    resolve(new ResponseResult(UpdateErrorCode.CannotProcessUpdate, `Update failed with error. ${error}`));
                                                }
                                            }));
                                            return yield updatePromise;
                                        }
                                        else {
                                            progressInfo.Percentage = 0;
                                            progressInfo.Status = UpdateStatus.InstallationError;
                                            updateProgress(progressInfo);
                                            return new ResponseResult(UpdateErrorCode.CannotProcessUpdate, `Execution failed!`);
                                        }
                                    }
                                    catch (err) {
                                        (_f = this._connection) === null || _f === void 0 ? void 0 : _f.SendAndWaitForResponseAsync(new ExecuteCommandRequest(`/runtime/scripts/restart_all.sh`), 12000, false);
                                        return new ResponseResult(UpdateErrorCode.CannotProcessUpdate, `Execution failed! Error ${err}`);
                                    }
                                }
                                return new ResponseResult(UpdateErrorCode.CannotProcessUpdate, `Execution failed! Cannot clear old update script!`);
                            }
                        }
                        return new ResponseResult(UpdateErrorCode.CannotProcessUpdate, `Execution failed! Cannot loggin using support account!`);
                    }
                default:
                    return new ResponseResult(UpdateErrorCode.CannotProcessUpdate, `Not supported update type, type: ${update.UpdateType}`);
            }
        });
    }
}
UpdatesService.ServiceName = "UpdatesService";
class CachedUpdateInfo extends UpdateInfo {
    constructor() {
        super(...arguments);
        this.UpdateIdentifier = "";
        this.ProviderName = "";
        this.UpdateType = null;
    }
}
export class ControllerUpdateNotAvailableInfo {
    constructor() {
        this.UpdatedResourceName = "";
        this.CurrentResourceVersion = "";
        this.UpdateType = null;
        this.ResponseResult = null;
    }
}
class ControllerUpdate {
    constructor() {
        this._updateIdentifier = "";
        this._updateResourceName = "";
        this._updateType = null;
        this._currentResourceVersion = "";
        this._newResourceVersion = "";
        this._updateDescription = new Map();
    }
    get UpdateDescription() {
        return this._updateDescription;
    }
    set UpdateDescription(value) {
        this._updateDescription = value;
    }
    get UpdateIdentifier() {
        return this._updateIdentifier;
    }
    get UpdatedResourceName() {
        return this._updateResourceName;
    }
    get UpdateType() {
        return this._updateType;
    }
    get CurrentResourceVersion() {
        return this._currentResourceVersion;
    }
    get NewResourceVersion() {
        return this._newResourceVersion;
    }
    set UpdateIdentifier(value) {
        this._updateIdentifier = value;
    }
    set UpdatedResourceName(value) {
        this._updateResourceName = value;
    }
    set UpdateType(value) {
        this._updateType = value;
    }
    set CurrentResourceVersion(value) {
        this._currentResourceVersion = value;
    }
    set NewResourceVersion(value) {
        this._newResourceVersion = value;
    }
}
class DeviceUpdate {
    constructor() {
        this._updatedResourceName = "";
        this._updateResourceGuid = "";
        this._protocol = "";
        this._protocolGuid = "";
        this._currentResourceVersion = "";
        this._newResourceVersion = "";
        this._isUpdateSafe = true;
        this._updateDescription = new Map();
    }
    get UpdateDescription() {
        return this._updateDescription;
    }
    set UpdateDescription(value) {
        this._updateDescription = value;
    }
    get UpdatedResourceName() {
        return this._updatedResourceName;
    }
    get UpdateResourceGuid() {
        return this._updateResourceGuid;
    }
    get Protocol() {
        return this._protocol;
    }
    get ProtocolGuid() {
        return this._protocolGuid;
    }
    get CurrentResourceVersion() {
        return this._currentResourceVersion;
    }
    get NewResourceVersion() {
        return this._newResourceVersion;
    }
    get IsUpdateSafe() {
        return this._isUpdateSafe;
    }
    set UpdatedResourceName(value) {
        this._updatedResourceName = value;
    }
    set UpdateResourceGuid(value) {
        this._updateResourceGuid = value;
    }
    set Protocol(value) {
        this._protocol = value;
    }
    set ProtocolGuid(value) {
        this._protocolGuid = value;
    }
    set CurrentResourceVersion(value) {
        this._currentResourceVersion = value;
    }
    set NewResourceVersion(value) {
        this._newResourceVersion = value;
    }
    set IsUpdateSafe(value) {
        this._isUpdateSafe = value;
    }
}
//Install update
class UpdateRequest extends DataFrame {
    constructor(uri) {
        super();
        this.Resource = "/updates/update/from/uri";
        this.Method = Method.Post;
        this.Data = uri;
    }
}
class UpdateStatusFrame {
    constructor() {
        this.ExpectedInstallationTime = 0;
        this.Guid = "";
        this.Package = new JsonPackageSchema();
        this.Progress = 0;
        this.State = UpdateState.PreparingSystem;
        this.UserInput = UserInput.Install;
    }
}
class JsonPackageSchema {
    constructor() {
        this.Package = new JsonPackage();
        this.Publisher = new Publisher();
    }
}
class JsonPackage {
    constructor() {
        this.CompatibleHardwareVersions = [];
        this.CompatibleOperatingSystems = [];
        this.CompatibleProcessorArch = [];
        this.CompatibleSoftwareVersions = [];
        this.DependencyRequiredExtensions = [];
        this.Description = new Map();
        this.Guid = "";
        this.Name = new Map();
        this.PackageAPIVersion = 0;
        this.PackageType = PackageType.Unknown;
        this.PackageVersion = 0;
        this.PublicationTime = Date.now().toString();
        this.PublisherGuid = "";
        this.RequiredDownloadDiskSpace = 0;
        this.RequiredInstallDiskSpace = 0;
        this.UpdateChannels = [];
    }
}
class Hardware {
    constructor() {
        this.Guid = "";
        this.MaxVersion = 0;
        this.MinVersion = 0;
    }
}
var OperatingSystem;
(function (OperatingSystem) {
    OperatingSystem[OperatingSystem["Windows"] = 0] = "Windows";
    OperatingSystem[OperatingSystem["Linux"] = 1] = "Linux";
    OperatingSystem[OperatingSystem["Unix"] = 2] = "Unix";
    OperatingSystem[OperatingSystem["OSX"] = 3] = "OSX";
    OperatingSystem[OperatingSystem["Android"] = 4] = "Android";
    OperatingSystem[OperatingSystem["iOS"] = 5] = "iOS";
    OperatingSystem[OperatingSystem["WindowsMobile"] = 6] = "WindowsMobile";
})(OperatingSystem || (OperatingSystem = {}));
var ProcessorArch;
(function (ProcessorArch) {
    ProcessorArch[ProcessorArch["Any"] = 0] = "Any";
    ProcessorArch[ProcessorArch["MIPS"] = 1] = "MIPS";
    ProcessorArch[ProcessorArch["ARM32"] = 2] = "ARM32";
    ProcessorArch[ProcessorArch["ARM64"] = 3] = "ARM64";
    ProcessorArch[ProcessorArch["IntelX86"] = 4] = "IntelX86";
    ProcessorArch[ProcessorArch["IntelX64"] = 5] = "IntelX64";
})(ProcessorArch || (ProcessorArch = {}));
class DependencyExtension {
    constructor() {
        this.CompatibleVersions = 0;
        this.ExtensionGuid = 0;
        this.MaximumCompatibleVersion = 0;
        this.MinimumCompatibleVersion = 0;
    }
}
var PackageType;
(function (PackageType) {
    PackageType[PackageType["Unknown"] = 0] = "Unknown";
    PackageType[PackageType["Environment"] = 1] = "Environment";
    PackageType[PackageType["OperatingSystem"] = 2] = "OperatingSystem";
    PackageType[PackageType["Application"] = 3] = "Application";
    PackageType[PackageType["Extension"] = 4] = "Extension";
    PackageType[PackageType["HardwareFirmware"] = 5] = "HardwareFirmware";
    PackageType[PackageType["Script"] = 6] = "Script";
    PackageType[PackageType["Command"] = 7] = "Command";
})(PackageType || (PackageType = {}));
class Publisher {
    constructor() {
        this.Description = "";
        this.Email = "";
        this.Guid = "";
        this.Name = "";
        this.Phone = "";
        this.PublisherGuid = "";
        this.Webpage = "";
    }
}
class InstallRequest extends DataFrame {
    constructor(uri) {
        super();
        this.Resource = "/updates/install/from/url";
        this.Method = Method.Put;
        this.Data = uri;
    }
}
class LoginUserRequest extends DataFrame {
    constructor(data) {
        super();
        this.Resource = "/users/user/login";
        this.Method = Method.Put;
        this.Data = data;
    }
}
class LoginUser {
    constructor() {
        this.Email = "";
        this.Password = "";
    }
}
class ExecuteCommandRequest extends DataFrame {
    constructor(command) {
        super();
        this.Resource = "/controller/runtime/execute";
        this.Method = Method.Post;
        this.Data = command;
    }
}
//# sourceMappingURL=UpdatesService.js.map